package com.smartgwt.sample.client;

import static com.smartgwt.client.util.JSOHelper.convertToJava;
import static com.smartgwt.sample.Constants.ERRORS_MESSAGE_KEY;
import static com.smartgwt.sample.Constants.GRID_UPDATES_CHANNEL;
import static com.smartgwt.sample.Constants.INVALIDATE_CACHE_MESSAGE_KEY;
import static com.smartgwt.sample.Constants.OPERATION_TYPE_MESSAGE_KEY;
import static com.smartgwt.sample.Constants.RECORDS_MESSAGE_KEY;

import java.util.List;
import java.util.Map;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.JavaScriptObject;
import com.smartgwt.client.data.DSRequest;
import com.smartgwt.client.data.DSResponse;
import com.smartgwt.client.data.DataSource;
import com.smartgwt.client.data.DataSourceField;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.rpc.Messaging;
import com.smartgwt.client.rpc.MessagingCallback;
import com.smartgwt.client.types.DSOperationType;
import com.smartgwt.client.types.FieldType;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.HTMLFlow;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.form.fields.FloatItem;
import com.smartgwt.client.widgets.form.fields.FormItem;
import com.smartgwt.client.widgets.form.fields.SelectItem;
import com.smartgwt.client.widgets.form.fields.TextItem;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridEditorContext;
import com.smartgwt.client.widgets.grid.ListGridEditorCustomizer;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.layout.VLayout;

public class RealTimeGridUpdates implements EntryPoint {

    private ListGrid listGrid;
    private DataSource ds;

    /**
     * {@inheritDoc}
     */
    public void onModuleLoad() {

        ds = DataSource.get("employees");

        final VLayout vlayout = new VLayout();
        vlayout.setMembersMargin(15);
        vlayout.setPadding(15);
        vlayout.setWidth100();
        vlayout.setHeight100();

        listGrid = new ListGrid();
        listGrid.setDataSource(ds);
        listGrid.setAutoFetchData(true);

        listGrid.setCanEdit(true);
        listGrid.setEditByCell(false);
        listGrid.setEditorCustomizer(new RealTimeListGridEditorCustomizer());
        listGrid.setCanRemoveRecords(true);
        listGrid.setAnimateRemoveRecord(true);

        listGrid.draw();

        final IButton addNewRecordButton = new IButton("Add New Record");
        addNewRecordButton.addClickHandler(new ClickHandler() {

            public void onClick(ClickEvent event) {
                listGrid.startEditingNew();
            }
        });
        addNewRecordButton.draw();

        vlayout.addMember(listGrid);
        vlayout.addMember(addNewRecordButton);
        vlayout.draw();

        Messaging.subscribe(GRID_UPDATES_CHANNEL, new MessagingCallback() {
            public void execute(Object data) {
                updateGrid((JavaScriptObject) data);
            }
        });
    }

    private void updateGrid(final JavaScriptObject dataObj) {

        // Create a fake DSRequest and DSResponse to send to the list grid's underlying
        // data source to force it to update its cache.

        final DSResponse dsResponse = new DSResponse();
        final DSRequest dsRequest = new DSRequest();

        @SuppressWarnings("unchecked")
        final Map<String, ?> data = (Map<String, ?>) convertToJava(dataObj);

        // Determine whether to invalidate cache
        dsResponse.setInvalidateCache((Boolean) data.get(INVALIDATE_CACHE_MESSAGE_KEY));

        // Determine the operation type
        final DSOperationType operationType =
                valueOfDSOperationType((String) data.get(OPERATION_TYPE_MESSAGE_KEY));
        dsRequest.setOperationType(operationType);

        // Construct the record data
        @SuppressWarnings("unchecked")
        List<Map<?, ?>> dataRecords = (List<Map<?, ?>>) data.get(RECORDS_MESSAGE_KEY);
        if (dataRecords != null && !dataRecords.isEmpty()) {

            final int dataRecordsSize = dataRecords.size();

            final Record[] records = new Record[dataRecordsSize];
            int i = 0;
            for (Map<?, ?> dataRecord : dataRecords) {
                records[i++] = new Record(dataRecord);
            }

            dsResponse.setData(records);
        }

        // Update the list grid's cache
        listGrid.getDataSource().updateCaches(dsResponse, dsRequest);
    }

    private static DSOperationType valueOfDSOperationType(final String value) {
        for (DSOperationType type : DSOperationType.values()) {
            if (type.getValue().equals(value)) {
                return type;
            }
        }
        return null;
    }

    class RealTimeListGridEditorCustomizer implements ListGridEditorCustomizer {

        public FormItem getEditor(ListGridEditorContext context) {
            final ListGridField field = context.getEditField();
            final String fieldName = field.getName();

            final DataSourceField dsField = ds.getField(fieldName);
            final FieldType dsFieldType = dsField.getType();

            if (dsFieldType == FieldType.TEXT) {
                @SuppressWarnings("unchecked")
                final Map<?, String> valueMap = (Map<?, String>) dsField.getValueMap();
                if (valueMap == null || valueMap.isEmpty()) {
                    final TextItem textItem = new TextItem();
                    textItem.setLength(dsField.getLength());
                    return textItem;
                } else {
                    final SelectItem selectItem = new SelectItem();
                    selectItem.setValueMap(valueMap.values().toArray(new String[0]));
                    return selectItem;
                }
            } else if (dsFieldType == FieldType.FLOAT) {
                return new FloatItem();
            } else {
                return null;
            }
        }

    }

}
