package com.smartgwt.sample.server;

import static com.smartgwt.sample.Constants.ERRORS_MESSAGE_KEY;
import static com.smartgwt.sample.Constants.GRID_UPDATES_CHANNEL;
import static com.smartgwt.sample.Constants.INVALIDATE_CACHE_MESSAGE_KEY;
import static com.smartgwt.sample.Constants.OPERATION_TYPE_MESSAGE_KEY;
import static com.smartgwt.sample.Constants.RECORDS_MESSAGE_KEY;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.isomorphic.datasource.DSRequest;
import com.isomorphic.datasource.DSResponse;
import com.isomorphic.messaging.ISCMessage;
import com.isomorphic.messaging.ISCMessageDispatcher;
import com.isomorphic.sql.SQLDataSource;
import com.isomorphic.util.ErrorReport;
import com.smartgwt.client.types.DSOperationType;

@SuppressWarnings("serial")
public final class GridUpdatesDataSource extends SQLDataSource {

    @Override
    public DSResponse executeAdd(DSRequest req) throws Exception {
        final DSResponse resp = super.executeAdd(req);
        sendToClients(resp, DSOperationType.ADD);
        return resp;
    }

    @Override
    public DSResponse executeRemove(DSRequest req) throws Exception {
        final DSResponse resp = super.executeRemove(req);
        sendToClients(resp, DSOperationType.REMOVE);
        return resp;
    }

    @Override
    public DSResponse executeUpdate(DSRequest req) throws Exception {
        final DSResponse resp = super.executeUpdate(req);
        sendToClients(resp, DSOperationType.UPDATE);
        return resp;
    }

    @SuppressWarnings({ "unchecked", "rawtypes" })
    private void sendToClients(final DSResponse resp, final DSOperationType operationType)
            throws Exception {

        final ISCMessageDispatcher dispatcher = ISCMessageDispatcher.instance();

        // Convert the data source response and the operation type into a form
        // usage by the client-side to construct a corresponding
        // com.smartgwt.client.data.DSResponse DSResponse
        // and com.smartgwt.client.data.DSRequest DSRequest.

        final Map data = new HashMap();

        data.put(INVALIDATE_CACHE_MESSAGE_KEY, resp.getInvalidateCache());
        data.put(OPERATION_TYPE_MESSAGE_KEY, operationType.getValue());
        data.put(RECORDS_MESSAGE_KEY, resp.getRecords());

        dispatcher.send(new ISCMessage(GRID_UPDATES_CHANNEL, (Object) data));
    }
}
